@@ -18,9 +18,9 @@ fit.bcart <- bcart(X=X, Z=Z, XX=XX, pred.n=FALSE, R=10,
                    BTE=c(2000,12000,2))
 rmse[1] <- sqrt(mean((fit.bcart$ZZ.mean - ff$Ytrue)^2))
 
-## dynamic treed linear model
-fit.pltc <- dynaTrees(X=X, y=Z, XX=XX, plotit=FALSE)
-rmse[2] <- sqrt(mean(apply(fit.pltc$mean, 1, mean) - ff$Ytrue)^2)
+## dynamic treed constant model
+fit.dtc <- dynaTrees(X=X, y=Z, XX=XX, plotit=FALSE)
+rmse[2] <- sqrt(mean(apply(fit.dtc$mean, 1, mean) - ff$Ytrue)^2)
   
 ## MCMC treed linear model
 fit.btlm <- btlm(X=X, Z=Z, XX=XX, pred.n=FALSE, R=10,
@@ -28,8 +28,8 @@ fit.btlm <- btlm(X=X, Z=Z, XX=XX, pred.n=FALSE, R=10,
 rmse[3] <- sqrt(mean((fit.btlm$ZZ.mean - ff$Ytrue)^2))
 
 ## dynamic treed linear model
-fit.pltl <- dynaTrees(X=X, y=Z, XX=XX, model="linear", plotit=FALSE)
-rmse[4] <- sqrt(mean(apply(fit.pltl$mean, 1, mean) - ff$Ytrue)^2)
+fit.dtl <- dynaTrees(X=X, y=Z, XX=XX, model="linear", plotit=FALSE)
+rmse[4] <- sqrt(mean(apply(fit.dtl$mean, 1, mean) - ff$Ytrue)^2)
 
 ## MCMC GP
 fit.bgp <- bgp(X=X, Z=Z, XX=XX, pred.n=FALSE, BTE=c(2000,12000,2))
